// Part of ImGui Bundle - MIT License - Copyright (c) 2022-2023 Pascal Thomet - https://github.com/pthom/imgui_bundle
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>
#include <pybind11/functional.h>
#include <pybind11/numpy.h>

#include "imgui-node-editor/imgui_node_editor_internal.h"
#include "imgui-node-editor/imgui_node_editor.h"
#include "imgui_node_editor_immapp/node_editor_default_context.h"


namespace py = pybind11;


namespace ax
{
    namespace NodeEditor
    {
        // using EditorContext = Detail::EditorContext;
        struct EditorContext: public Detail::EditorContext {};
    }
}


namespace
{
    uintptr_t _next_id = 1;

    uintptr_t get_next_id()
    {
        uintptr_t r = _next_id;
        ++ _next_id;
        return r;
    }
}

PYBIND11_MAKE_OPAQUE(ax::NodeEditor::Detail::EditorContext);
PYBIND11_MAKE_OPAQUE(ax::NodeEditor::EditorContext);


// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  AUTOGENERATED CODE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// <litgen_glue_code>  // Autogenerated code below! Do not edit!

// </litgen_glue_code> // Autogenerated code end
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  AUTOGENERATED CODE END !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


void py_init_module_imgui_node_editor(py::module& m)
{
    using namespace ax::NodeEditor;
    using CanvasSizeModeAlias = ax::NodeEditor::CanvasSizeMode;


    py::class_<EditorContext>(m, "EditorContext");

    py::class_<NodeId>(m, "NodeId")
        .def(py::init<>())
        .def(py::init<uintptr_t>())
        .def("create", []() { return NodeId(get_next_id()); })
        .def("id", [](const NodeId& self) { return self.Get(); })
        .def("__eq__", [](const NodeId& self, const NodeId& other) {
            return self.Get() == other.Get();
        })
        .def("__str__", [](const NodeId& self) { return std::to_string(self.Get()); })
        ;
    py::class_<LinkId>(m, "LinkId")
        .def(py::init<>())
        .def(py::init<uintptr_t>())
        .def("create", []() { return LinkId(get_next_id()); })
        .def("id", [](const LinkId& self) { return self.Get(); })
        .def("__eq__", [](const LinkId& self, const LinkId& other) {
            return self.Get() == other.Get();
        })
        .def("__str__", [](const LinkId& self) { return std::to_string(self.Get()); })
        ;
    py::class_<PinId>(m, "PinId")
        .def(py::init<>())
        .def(py::init<uintptr_t>())
        .def("create", []() { return PinId(get_next_id()); })
        .def("id", [](const PinId& self) { return self.Get(); })
        .def("__eq__", [](const PinId& self, const PinId& other) {
            return self.Get() == other.Get();
        })
        .def("__str__", [](const PinId& self) { return std::to_string(self.Get()); })
        ;

    m.def("suspend_editor_canvas", []() {
       GetCurrentEditor()->Suspend();
    });
    m.def("resume_editor_canvas", []() {
        GetCurrentEditor()->Resume();
    });

    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  AUTOGENERATED CODE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    // <litgen_pydef> // Autogenerated code below! Do not edit!
    ////////////////////    <generated_from:imgui_node_editor.h>    ////////////////////
    // # ifndef __IMGUI_NODE_EDITOR_H__
    //


    py::enum_<ax::NodeEditor::PinKind>(m, "PinKind", py::arithmetic(), "------------------------------------------------------------------------------")    // imgui_node_editor.h:47
        .value("input", ax::NodeEditor::PinKind::Input, "")
        .value("output", ax::NodeEditor::PinKind::Output, "");


    py::enum_<ax::NodeEditor::FlowDirection>(m, "FlowDirection", py::arithmetic(), "")    // imgui_node_editor.h:53
        .value("forward", ax::NodeEditor::FlowDirection::Forward, "")
        .value("backward", ax::NodeEditor::FlowDirection::Backward, "");


    py::enum_<ax::NodeEditor::CanvasSizeMode>(m, "CanvasSizeMode", py::arithmetic(), "")    // imgui_node_editor.h:59
        .value("fit_vertical_view", ax::NodeEditor::CanvasSizeMode::FitVerticalView, "Previous view will be scaled to fit new view on Y axis")
        .value("fit_horizontal_view", ax::NodeEditor::CanvasSizeMode::FitHorizontalView, "Previous view will be scaled to fit new view on X axis")
        .value("center_only", ax::NodeEditor::CanvasSizeMode::CenterOnly, "Previous view will be centered on new view");


    py::enum_<ax::NodeEditor::SaveReasonFlags>(m, "SaveReasonFlags", py::arithmetic(), "------------------------------------------------------------------------------")    // imgui_node_editor.h:68
        .value("none", ax::NodeEditor::SaveReasonFlags::None, "")
        .value("navigation", ax::NodeEditor::SaveReasonFlags::Navigation, "")
        .value("position", ax::NodeEditor::SaveReasonFlags::Position, "")
        .value("size", ax::NodeEditor::SaveReasonFlags::Size, "")
        .value("selection", ax::NodeEditor::SaveReasonFlags::Selection, "")
        .value("add_node", ax::NodeEditor::SaveReasonFlags::AddNode, "")
        .value("remove_node", ax::NodeEditor::SaveReasonFlags::RemoveNode, "")
        .value("user", ax::NodeEditor::SaveReasonFlags::User, "");


    auto pyClassConfig =
        py::class_<ax::NodeEditor::Config>    // imgui_node_editor.h:91
            (m, "Config", "")
        .def_readwrite("settings_file", &Config::SettingsFile, "")    // imgui_node_editor.h:95
        .def_readwrite("user_pointer", &Config::UserPointer, "")    // imgui_node_editor.h:102
        .def_readwrite("canvas_size_mode", &Config::CanvasSizeMode, "")    // imgui_node_editor.h:104
        .def_readwrite("drag_button_index", &Config::DragButtonIndex, "Mouse button index drag action will react to (0-left, 1-right, 2-middle)")    // imgui_node_editor.h:105
        .def_readwrite("select_button_index", &Config::SelectButtonIndex, "Mouse button index select action will react to (0-left, 1-right, 2-middle)")    // imgui_node_editor.h:106
        .def_readwrite("navigate_button_index", &Config::NavigateButtonIndex, "Mouse button index navigate action will react to (0-left, 1-right, 2-middle)")    // imgui_node_editor.h:107
        .def_readwrite("context_menu_button_index", &Config::ContextMenuButtonIndex, "Mouse button index context menu action will react to (0-left, 1-right, 2-middle)")    // imgui_node_editor.h:108
        .def(py::init<>())    // imgui_node_editor.h:110
        ;


    py::enum_<ax::NodeEditor::StyleColor>(m, "StyleColor", py::arithmetic(), "------------------------------------------------------------------------------")    // imgui_node_editor.h:131
        .value("bg", ax::NodeEditor::StyleColor_Bg, "")
        .value("grid", ax::NodeEditor::StyleColor_Grid, "")
        .value("node_bg", ax::NodeEditor::StyleColor_NodeBg, "")
        .value("node_border", ax::NodeEditor::StyleColor_NodeBorder, "")
        .value("hov_node_border", ax::NodeEditor::StyleColor_HovNodeBorder, "")
        .value("sel_node_border", ax::NodeEditor::StyleColor_SelNodeBorder, "")
        .value("node_sel_rect", ax::NodeEditor::StyleColor_NodeSelRect, "")
        .value("node_sel_rect_border", ax::NodeEditor::StyleColor_NodeSelRectBorder, "")
        .value("hov_link_border", ax::NodeEditor::StyleColor_HovLinkBorder, "")
        .value("sel_link_border", ax::NodeEditor::StyleColor_SelLinkBorder, "")
        .value("highlight_link_border", ax::NodeEditor::StyleColor_HighlightLinkBorder, "")
        .value("link_sel_rect", ax::NodeEditor::StyleColor_LinkSelRect, "")
        .value("link_sel_rect_border", ax::NodeEditor::StyleColor_LinkSelRectBorder, "")
        .value("pin_rect", ax::NodeEditor::StyleColor_PinRect, "")
        .value("pin_rect_border", ax::NodeEditor::StyleColor_PinRectBorder, "")
        .value("flow", ax::NodeEditor::StyleColor_Flow, "")
        .value("flow_marker", ax::NodeEditor::StyleColor_FlowMarker, "")
        .value("group_bg", ax::NodeEditor::StyleColor_GroupBg, "")
        .value("group_border", ax::NodeEditor::StyleColor_GroupBorder, "")
        .value("count", ax::NodeEditor::StyleColor_Count, "");


    py::enum_<ax::NodeEditor::StyleVar>(m, "StyleVar", py::arithmetic(), "")    // imgui_node_editor.h:156
        .value("node_padding", ax::NodeEditor::StyleVar_NodePadding, "")
        .value("node_rounding", ax::NodeEditor::StyleVar_NodeRounding, "")
        .value("node_border_width", ax::NodeEditor::StyleVar_NodeBorderWidth, "")
        .value("hovered_node_border_width", ax::NodeEditor::StyleVar_HoveredNodeBorderWidth, "")
        .value("selected_node_border_width", ax::NodeEditor::StyleVar_SelectedNodeBorderWidth, "")
        .value("pin_rounding", ax::NodeEditor::StyleVar_PinRounding, "")
        .value("pin_border_width", ax::NodeEditor::StyleVar_PinBorderWidth, "")
        .value("link_strength", ax::NodeEditor::StyleVar_LinkStrength, "")
        .value("source_direction", ax::NodeEditor::StyleVar_SourceDirection, "")
        .value("target_direction", ax::NodeEditor::StyleVar_TargetDirection, "")
        .value("scroll_duration", ax::NodeEditor::StyleVar_ScrollDuration, "")
        .value("flow_marker_distance", ax::NodeEditor::StyleVar_FlowMarkerDistance, "")
        .value("flow_speed", ax::NodeEditor::StyleVar_FlowSpeed, "")
        .value("flow_duration", ax::NodeEditor::StyleVar_FlowDuration, "")
        .value("pivot_alignment", ax::NodeEditor::StyleVar_PivotAlignment, "")
        .value("pivot_size", ax::NodeEditor::StyleVar_PivotSize, "")
        .value("pivot_scale", ax::NodeEditor::StyleVar_PivotScale, "")
        .value("pin_corners", ax::NodeEditor::StyleVar_PinCorners, "")
        .value("pin_radius", ax::NodeEditor::StyleVar_PinRadius, "")
        .value("pin_arrow_size", ax::NodeEditor::StyleVar_PinArrowSize, "")
        .value("pin_arrow_width", ax::NodeEditor::StyleVar_PinArrowWidth, "")
        .value("group_rounding", ax::NodeEditor::StyleVar_GroupRounding, "")
        .value("group_border_width", ax::NodeEditor::StyleVar_GroupBorderWidth, "")
        .value("highlight_connected_links", ax::NodeEditor::StyleVar_HighlightConnectedLinks, "")
        .value("snap_link_to_pin_dir", ax::NodeEditor::StyleVar_SnapLinkToPinDir, "")
        .value("hovered_node_border_offset", ax::NodeEditor::StyleVar_HoveredNodeBorderOffset, "")
        .value("selected_node_border_offset", ax::NodeEditor::StyleVar_SelectedNodeBorderOffset, "")
        .value("count", ax::NodeEditor::StyleVar_Count, "");


    auto pyClassStyle =
        py::class_<ax::NodeEditor::Style>    // imgui_node_editor.h:189
            (m, "Style", "")
        .def_readwrite("node_padding", &Style::NodePadding, "")    // imgui_node_editor.h:191
        .def_readwrite("node_rounding", &Style::NodeRounding, "")    // imgui_node_editor.h:192
        .def_readwrite("node_border_width", &Style::NodeBorderWidth, "")    // imgui_node_editor.h:193
        .def_readwrite("hovered_node_border_width", &Style::HoveredNodeBorderWidth, "")    // imgui_node_editor.h:194
        .def_readwrite("hover_node_border_offset", &Style::HoverNodeBorderOffset, "")    // imgui_node_editor.h:195
        .def_readwrite("selected_node_border_width", &Style::SelectedNodeBorderWidth, "")    // imgui_node_editor.h:196
        .def_readwrite("selected_node_border_offset", &Style::SelectedNodeBorderOffset, "")    // imgui_node_editor.h:197
        .def_readwrite("pin_rounding", &Style::PinRounding, "")    // imgui_node_editor.h:198
        .def_readwrite("pin_border_width", &Style::PinBorderWidth, "")    // imgui_node_editor.h:199
        .def_readwrite("link_strength", &Style::LinkStrength, "")    // imgui_node_editor.h:200
        .def_readwrite("source_direction", &Style::SourceDirection, "")    // imgui_node_editor.h:201
        .def_readwrite("target_direction", &Style::TargetDirection, "")    // imgui_node_editor.h:202
        .def_readwrite("scroll_duration", &Style::ScrollDuration, "")    // imgui_node_editor.h:203
        .def_readwrite("flow_marker_distance", &Style::FlowMarkerDistance, "")    // imgui_node_editor.h:204
        .def_readwrite("flow_speed", &Style::FlowSpeed, "")    // imgui_node_editor.h:205
        .def_readwrite("flow_duration", &Style::FlowDuration, "")    // imgui_node_editor.h:206
        .def_readwrite("pivot_alignment", &Style::PivotAlignment, "")    // imgui_node_editor.h:207
        .def_readwrite("pivot_size", &Style::PivotSize, "")    // imgui_node_editor.h:208
        .def_readwrite("pivot_scale", &Style::PivotScale, "")    // imgui_node_editor.h:209
        .def_readwrite("pin_corners", &Style::PinCorners, "")    // imgui_node_editor.h:210
        .def_readwrite("pin_radius", &Style::PinRadius, "")    // imgui_node_editor.h:211
        .def_readwrite("pin_arrow_size", &Style::PinArrowSize, "")    // imgui_node_editor.h:212
        .def_readwrite("pin_arrow_width", &Style::PinArrowWidth, "")    // imgui_node_editor.h:213
        .def_readwrite("group_rounding", &Style::GroupRounding, "")    // imgui_node_editor.h:214
        .def_readwrite("group_border_width", &Style::GroupBorderWidth, "")    // imgui_node_editor.h:215
        .def_readwrite("highlight_connected_links", &Style::HighlightConnectedLinks, "")    // imgui_node_editor.h:216
        .def_readwrite("snap_link_to_pin_dir", &Style::SnapLinkToPinDir, "when True link will start on the line defined by pin direction")    // imgui_node_editor.h:217
        .def(py::init<>())    // imgui_node_editor.h:220
        ;


    m.def("set_current_editor",    // imgui_node_editor.h:282
        ax::NodeEditor::SetCurrentEditor, py::arg("ctx"));

    m.def("get_current_editor",    // imgui_node_editor.h:283
        ax::NodeEditor::GetCurrentEditor, pybind11::return_value_policy::reference);

    m.def("create_editor",    // imgui_node_editor.h:284
        ax::NodeEditor::CreateEditor,
        py::arg("config") = py::none(),
        pybind11::return_value_policy::reference);

    m.def("destroy_editor",    // imgui_node_editor.h:285
        ax::NodeEditor::DestroyEditor, py::arg("ctx"));

    m.def("get_config",    // imgui_node_editor.h:286
        ax::NodeEditor::GetConfig,
        py::arg("ctx") = py::none(),
        pybind11::return_value_policy::reference);

    m.def("get_style",    // imgui_node_editor.h:288
        ax::NodeEditor::GetStyle, pybind11::return_value_policy::reference);

    m.def("get_style_color_name",    // imgui_node_editor.h:289
        ax::NodeEditor::GetStyleColorName,
        py::arg("color_index"),
        pybind11::return_value_policy::reference);

    m.def("push_style_color",    // imgui_node_editor.h:291
        ax::NodeEditor::PushStyleColor, py::arg("color_index"), py::arg("color"));

    m.def("pop_style_color",    // imgui_node_editor.h:292
        ax::NodeEditor::PopStyleColor, py::arg("count") = 1);

    m.def("push_style_var",    // imgui_node_editor.h:294
        py::overload_cast<ax::NodeEditor::StyleVar, float>(ax::NodeEditor::PushStyleVar), py::arg("var_index"), py::arg("value"));

    m.def("push_style_var",    // imgui_node_editor.h:295
        py::overload_cast<ax::NodeEditor::StyleVar, const ImVec2 &>(ax::NodeEditor::PushStyleVar), py::arg("var_index"), py::arg("value"));

    m.def("push_style_var",    // imgui_node_editor.h:296
        py::overload_cast<ax::NodeEditor::StyleVar, const ImVec4 &>(ax::NodeEditor::PushStyleVar), py::arg("var_index"), py::arg("value"));

    m.def("pop_style_var",    // imgui_node_editor.h:297
        ax::NodeEditor::PopStyleVar, py::arg("count") = 1);

    m.def("begin",    // imgui_node_editor.h:299
        ax::NodeEditor::Begin, py::arg("id"), py::arg("size") = ImVec2(0, 0));

    m.def("end",    // imgui_node_editor.h:300
        ax::NodeEditor::End);

    m.def("begin_node",    // imgui_node_editor.h:302
        ax::NodeEditor::BeginNode, py::arg("id"));

    m.def("begin_pin",    // imgui_node_editor.h:303
        ax::NodeEditor::BeginPin, py::arg("id"), py::arg("kind"));

    m.def("pin_rect",    // imgui_node_editor.h:304
        ax::NodeEditor::PinRect, py::arg("a"), py::arg("b"));

    m.def("pin_pivot_rect",    // imgui_node_editor.h:305
        ax::NodeEditor::PinPivotRect, py::arg("a"), py::arg("b"));

    m.def("pin_pivot_size",    // imgui_node_editor.h:306
        ax::NodeEditor::PinPivotSize, py::arg("size"));

    m.def("pin_pivot_scale",    // imgui_node_editor.h:307
        ax::NodeEditor::PinPivotScale, py::arg("scale"));

    m.def("pin_pivot_alignment",    // imgui_node_editor.h:308
        ax::NodeEditor::PinPivotAlignment, py::arg("alignment"));

    m.def("end_pin",    // imgui_node_editor.h:309
        ax::NodeEditor::EndPin);

    m.def("group",    // imgui_node_editor.h:310
        ax::NodeEditor::Group, py::arg("size"));

    m.def("end_node",    // imgui_node_editor.h:311
        ax::NodeEditor::EndNode);

    m.def("begin_group_hint",    // imgui_node_editor.h:313
        ax::NodeEditor::BeginGroupHint, py::arg("node_id"));

    m.def("get_group_min",    // imgui_node_editor.h:314
        ax::NodeEditor::GetGroupMin);

    m.def("get_group_max",    // imgui_node_editor.h:315
        ax::NodeEditor::GetGroupMax);

    m.def("get_hint_foreground_draw_list",    // imgui_node_editor.h:316
        ax::NodeEditor::GetHintForegroundDrawList, pybind11::return_value_policy::reference);

    m.def("get_hint_background_draw_list",    // imgui_node_editor.h:317
        ax::NodeEditor::GetHintBackgroundDrawList, pybind11::return_value_policy::reference);

    m.def("end_group_hint",    // imgui_node_editor.h:318
        ax::NodeEditor::EndGroupHint);

    m.def("get_node_background_draw_list",    // imgui_node_editor.h:321
        ax::NodeEditor::GetNodeBackgroundDrawList,
        py::arg("node_id"),
        "TODO: Add a way to manage node background channels",
        pybind11::return_value_policy::reference);

    m.def("link",    // imgui_node_editor.h:323
        ax::NodeEditor::Link, py::arg("id"), py::arg("start_pin_id"), py::arg("end_pin_id"), py::arg("color") = ImVec4(1, 1, 1, 1), py::arg("thickness") = 1.0f);

    m.def("flow",    // imgui_node_editor.h:325
        ax::NodeEditor::Flow, py::arg("link_id"), py::arg("direction") = ax::NodeEditor::FlowDirection::Forward);

    m.def("begin_create",    // imgui_node_editor.h:327
        ax::NodeEditor::BeginCreate, py::arg("color") = ImVec4(1, 1, 1, 1), py::arg("thickness") = 1.0f);

    m.def("query_new_link",    // imgui_node_editor.h:328
        py::overload_cast<ax::NodeEditor::PinId *, ax::NodeEditor::PinId *>(ax::NodeEditor::QueryNewLink), py::arg("start_id"), py::arg("end_id"));

    m.def("query_new_link",    // imgui_node_editor.h:329
        py::overload_cast<ax::NodeEditor::PinId *, ax::NodeEditor::PinId *, const ImVec4 &, float>(ax::NodeEditor::QueryNewLink), py::arg("start_id"), py::arg("end_id"), py::arg("color"), py::arg("thickness") = 1.0f);

    m.def("query_new_node",    // imgui_node_editor.h:330
        py::overload_cast<ax::NodeEditor::PinId *>(ax::NodeEditor::QueryNewNode), py::arg("pin_id"));

    m.def("query_new_node",    // imgui_node_editor.h:331
        py::overload_cast<ax::NodeEditor::PinId *, const ImVec4 &, float>(ax::NodeEditor::QueryNewNode), py::arg("pin_id"), py::arg("color"), py::arg("thickness") = 1.0f);

    m.def("accept_new_item",    // imgui_node_editor.h:332
        py::overload_cast<>(ax::NodeEditor::AcceptNewItem));

    m.def("accept_new_item",    // imgui_node_editor.h:333
        py::overload_cast<const ImVec4 &, float>(ax::NodeEditor::AcceptNewItem), py::arg("color"), py::arg("thickness") = 1.0f);

    m.def("reject_new_item",    // imgui_node_editor.h:334
        py::overload_cast<>(ax::NodeEditor::RejectNewItem));

    m.def("reject_new_item",    // imgui_node_editor.h:335
        py::overload_cast<const ImVec4 &, float>(ax::NodeEditor::RejectNewItem), py::arg("color"), py::arg("thickness") = 1.0f);

    m.def("end_create",    // imgui_node_editor.h:336
        ax::NodeEditor::EndCreate);

    m.def("begin_delete",    // imgui_node_editor.h:338
        ax::NodeEditor::BeginDelete);

    m.def("query_deleted_link",    // imgui_node_editor.h:339
        ax::NodeEditor::QueryDeletedLink, py::arg("link_id"), py::arg("start_id") = py::none(), py::arg("end_id") = py::none());

    m.def("query_deleted_node",    // imgui_node_editor.h:340
        ax::NodeEditor::QueryDeletedNode, py::arg("node_id"));

    m.def("accept_deleted_item",    // imgui_node_editor.h:341
        ax::NodeEditor::AcceptDeletedItem, py::arg("delete_dependencies") = true);

    m.def("reject_deleted_item",    // imgui_node_editor.h:342
        ax::NodeEditor::RejectDeletedItem);

    m.def("end_delete",    // imgui_node_editor.h:343
        ax::NodeEditor::EndDelete);

    m.def("set_node_position",    // imgui_node_editor.h:345
        ax::NodeEditor::SetNodePosition, py::arg("node_id"), py::arg("editor_position"));

    m.def("set_group_size",    // imgui_node_editor.h:346
        ax::NodeEditor::SetGroupSize, py::arg("node_id"), py::arg("size"));

    m.def("get_node_position",    // imgui_node_editor.h:347
        ax::NodeEditor::GetNodePosition, py::arg("node_id"));

    m.def("get_node_size",    // imgui_node_editor.h:348
        ax::NodeEditor::GetNodeSize, py::arg("node_id"));

    m.def("center_node_on_screen",    // imgui_node_editor.h:349
        ax::NodeEditor::CenterNodeOnScreen, py::arg("node_id"));

    m.def("set_node_z_position",    // imgui_node_editor.h:350
        ax::NodeEditor::SetNodeZPosition,
        py::arg("node_id"), py::arg("z"),
        "Sets node z position, nodes with higher value are drawn over nodes with lower value");

    m.def("get_node_z_position",    // imgui_node_editor.h:351
        ax::NodeEditor::GetNodeZPosition,
        py::arg("node_id"),
        "Returns node z position, defaults is 0.0");

    m.def("restore_node_state",    // imgui_node_editor.h:353
        ax::NodeEditor::RestoreNodeState, py::arg("node_id"));

    m.def("suspend",    // imgui_node_editor.h:355
        ax::NodeEditor::Suspend);

    m.def("resume",    // imgui_node_editor.h:356
        ax::NodeEditor::Resume);

    m.def("is_suspended",    // imgui_node_editor.h:357
        ax::NodeEditor::IsSuspended);

    m.def("is_active",    // imgui_node_editor.h:359
        ax::NodeEditor::IsActive);

    m.def("has_selection_changed",    // imgui_node_editor.h:361
        ax::NodeEditor::HasSelectionChanged);

    m.def("get_selected_object_count",    // imgui_node_editor.h:362
        ax::NodeEditor::GetSelectedObjectCount);

    m.def("get_selected_nodes",    // imgui_node_editor.h:363
        ax::NodeEditor::GetSelectedNodes, py::arg("nodes"), py::arg("size"));

    m.def("get_selected_links",    // imgui_node_editor.h:364
        ax::NodeEditor::GetSelectedLinks, py::arg("links"), py::arg("size"));

    m.def("is_node_selected",    // imgui_node_editor.h:365
        ax::NodeEditor::IsNodeSelected, py::arg("node_id"));

    m.def("is_link_selected",    // imgui_node_editor.h:366
        ax::NodeEditor::IsLinkSelected, py::arg("link_id"));

    m.def("clear_selection",    // imgui_node_editor.h:367
        ax::NodeEditor::ClearSelection);

    m.def("select_node",    // imgui_node_editor.h:368
        ax::NodeEditor::SelectNode, py::arg("node_id"), py::arg("append") = false);

    m.def("select_link",    // imgui_node_editor.h:369
        ax::NodeEditor::SelectLink, py::arg("link_id"), py::arg("append") = false);

    m.def("deselect_node",    // imgui_node_editor.h:370
        ax::NodeEditor::DeselectNode, py::arg("node_id"));

    m.def("deselect_link",    // imgui_node_editor.h:371
        ax::NodeEditor::DeselectLink, py::arg("link_id"));

    m.def("delete_node",    // imgui_node_editor.h:373
        ax::NodeEditor::DeleteNode, py::arg("node_id"));

    m.def("delete_link",    // imgui_node_editor.h:374
        ax::NodeEditor::DeleteLink, py::arg("link_id"));

    m.def("has_any_links",    // imgui_node_editor.h:376
        py::overload_cast<ax::NodeEditor::NodeId>(ax::NodeEditor::HasAnyLinks),
        py::arg("node_id"),
        "Returns True if node has any link connected");

    m.def("has_any_links",    // imgui_node_editor.h:377
        py::overload_cast<ax::NodeEditor::PinId>(ax::NodeEditor::HasAnyLinks),
        py::arg("pin_id"),
        "Return True if pin has any link connected");

    m.def("break_links",    // imgui_node_editor.h:378
        py::overload_cast<ax::NodeEditor::NodeId>(ax::NodeEditor::BreakLinks),
        py::arg("node_id"),
        "Break all links connected to this node");

    m.def("break_links",    // imgui_node_editor.h:379
        py::overload_cast<ax::NodeEditor::PinId>(ax::NodeEditor::BreakLinks),
        py::arg("pin_id"),
        "Break all links connected to this pin");

    m.def("navigate_to_content",    // imgui_node_editor.h:381
        ax::NodeEditor::NavigateToContent, py::arg("duration") = -1);

    m.def("navigate_to_selection",    // imgui_node_editor.h:382
        ax::NodeEditor::NavigateToSelection, py::arg("zoom_in") = false, py::arg("duration") = -1);

    m.def("show_node_context_menu",    // imgui_node_editor.h:384
        ax::NodeEditor::ShowNodeContextMenu, py::arg("node_id"));

    m.def("show_pin_context_menu",    // imgui_node_editor.h:385
        ax::NodeEditor::ShowPinContextMenu, py::arg("pin_id"));

    m.def("show_link_context_menu",    // imgui_node_editor.h:386
        ax::NodeEditor::ShowLinkContextMenu, py::arg("link_id"));

    m.def("show_background_context_menu",    // imgui_node_editor.h:387
        ax::NodeEditor::ShowBackgroundContextMenu);

    m.def("enable_shortcuts",    // imgui_node_editor.h:389
        ax::NodeEditor::EnableShortcuts, py::arg("enable"));

    m.def("are_shortcuts_enabled",    // imgui_node_editor.h:390
        ax::NodeEditor::AreShortcutsEnabled);

    m.def("begin_shortcut",    // imgui_node_editor.h:392
        ax::NodeEditor::BeginShortcut);

    m.def("accept_cut",    // imgui_node_editor.h:393
        ax::NodeEditor::AcceptCut);

    m.def("accept_copy",    // imgui_node_editor.h:394
        ax::NodeEditor::AcceptCopy);

    m.def("accept_paste",    // imgui_node_editor.h:395
        ax::NodeEditor::AcceptPaste);

    m.def("accept_duplicate",    // imgui_node_editor.h:396
        ax::NodeEditor::AcceptDuplicate);

    m.def("accept_create_node",    // imgui_node_editor.h:397
        ax::NodeEditor::AcceptCreateNode);

    m.def("get_action_context_size",    // imgui_node_editor.h:398
        ax::NodeEditor::GetActionContextSize);

    m.def("get_action_context_nodes",    // imgui_node_editor.h:399
        ax::NodeEditor::GetActionContextNodes, py::arg("nodes"), py::arg("size"));

    m.def("get_action_context_links",    // imgui_node_editor.h:400
        ax::NodeEditor::GetActionContextLinks, py::arg("links"), py::arg("size"));

    m.def("end_shortcut",    // imgui_node_editor.h:401
        ax::NodeEditor::EndShortcut);

    m.def("get_current_zoom",    // imgui_node_editor.h:403
        ax::NodeEditor::GetCurrentZoom);

    m.def("get_hovered_node",    // imgui_node_editor.h:405
        ax::NodeEditor::GetHoveredNode);

    m.def("get_hovered_pin",    // imgui_node_editor.h:406
        ax::NodeEditor::GetHoveredPin);

    m.def("get_hovered_link",    // imgui_node_editor.h:407
        ax::NodeEditor::GetHoveredLink);

    m.def("get_double_clicked_node",    // imgui_node_editor.h:408
        ax::NodeEditor::GetDoubleClickedNode);

    m.def("get_double_clicked_pin",    // imgui_node_editor.h:409
        ax::NodeEditor::GetDoubleClickedPin);

    m.def("get_double_clicked_link",    // imgui_node_editor.h:410
        ax::NodeEditor::GetDoubleClickedLink);

    m.def("is_background_clicked",    // imgui_node_editor.h:411
        ax::NodeEditor::IsBackgroundClicked);

    m.def("is_background_double_clicked",    // imgui_node_editor.h:412
        ax::NodeEditor::IsBackgroundDoubleClicked);

    m.def("get_background_click_button_index",    // imgui_node_editor.h:413
        ax::NodeEditor::GetBackgroundClickButtonIndex, "-1 if none");

    m.def("get_background_double_click_button_index",    // imgui_node_editor.h:414
        ax::NodeEditor::GetBackgroundDoubleClickButtonIndex, "-1 if none");

    m.def("get_link_pins",    // imgui_node_editor.h:416
        ax::NodeEditor::GetLinkPins,
        py::arg("link_id"), py::arg("start_pin_id"), py::arg("end_pin_id"),
        "pass None if particular pin do not interest you");

    m.def("pin_had_any_links",    // imgui_node_editor.h:418
        ax::NodeEditor::PinHadAnyLinks, py::arg("pin_id"));

    m.def("get_screen_size",    // imgui_node_editor.h:420
        ax::NodeEditor::GetScreenSize);

    m.def("screen_to_canvas",    // imgui_node_editor.h:421
        ax::NodeEditor::ScreenToCanvas, py::arg("pos"));

    m.def("canvas_to_screen",    // imgui_node_editor.h:422
        ax::NodeEditor::CanvasToScreen, py::arg("pos"));

    m.def("get_node_count",    // imgui_node_editor.h:424
        ax::NodeEditor::GetNodeCount, "Returns number of submitted nodes since Begin() call");

    m.def("get_ordered_node_ids",    // imgui_node_editor.h:425
        ax::NodeEditor::GetOrderedNodeIds,
        py::arg("nodes"), py::arg("size"),
        "Fills an array with node id's in order they're drawn; up to 'size` elements are set. Returns actual size of filled id's.");
    // # endif
    ////////////////////    </generated_from:imgui_node_editor.h>    ////////////////////


    ////////////////////    <generated_from:node_editor_default_context.h>    ////////////////////
    m.def("default_node_editor_context_immapp",    // node_editor_default_context.h:9
        DefaultNodeEditorContext_Immapp, pybind11::return_value_policy::reference);

    m.def("suspend_node_editor_canvas_immapp",    // node_editor_default_context.h:10
        SuspendNodeEditorCanvas_Immapp);

    m.def("resume_node_editor_canvas_immapp",    // node_editor_default_context.h:11
        ResumeNodeEditorCanvas_Immapp);
    ////////////////////    </generated_from:node_editor_default_context.h>    ////////////////////

    // </litgen_pydef> // Autogenerated code end
    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  AUTOGENERATED CODE END !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
}
